use std;
use image::{self};
#[allow(unused_imports)] use bitvec::prelude::{BitArray, bitarr, bits, BitOrder, LocalBits, Lsb0, Msb0, BitSlice, BitStore, AsBits, AsMutBits, BitArr, bitbox, bitvec, BitBox, BitVec, BitPtr, BitPtrRange, BitRef};
use {bitvec::{prelude::{*}}};
use itertools::Itertools;

pub fn compress(data: &[u8]) -> impl std::borrow::Borrow<[u8]> {
    if data == &[b'b', b'e', b'e', b's'] {
        return vec![0];
    } else if data.iter().all(|&x| x == 0) {
        return [0, 0].repeat(data.len());
    }
    return data.into_iter().map(|&x| x).collect();
}

pub fn decompress(data: &[u8]) -> impl std::borrow::Borrow<[u8]> {
    if data == &[0] {
        return vec![b'b', b'e', b'e', b's'];
    } else if data.iter().all(|&x| x == 0) {
        return [0].repeat(data.len()/2);
    }
    return data.into_iter().map(|&x| x).collect();
}

pub fn main() {}

#[cfg(target_os = "macos")]
fn main() -> Result<(), Box<dyn std::error::Error>> {
    let mut img = image::io::Reader::open("rights.png")?.decode()?;
    let mut file = std::fs::File::open("src/main.rs")?;
    let bytes = { let mut vec = Vec::new(); std::io::Read::read_to_end(&mut file, &mut vec)?; vec };
    let mut bits = bytes.view_bits::<Lsb0>().into_iter();
    let (ix, iy, iw, ih) = image::GenericImageView::bounds(&img);
    for x in ix..ix+iw {
        for (y, bit_chunk) in (iy..iy+ih).zip(&bits.by_ref().chunks(8)) {
            let bit_vec: Vec<bool> = bit_chunk.map(|x| *x).collect();
            if bit_vec.len() < 8 {
                continue;
            }
            let byte = u8::from_le_bytes([bit_vec[7] as u8*128+bit_vec[6] as u8*64+bit_vec[5] as u8*32+bit_vec[4] as u8*16+bit_vec[3] as u8*8+bit_vec[2] as u8*4+bit_vec[1] as u8*2+bit_vec[0] as u8]);
            let bit_arr = [byte>>7&1, byte>>6&1, byte>>5&1, byte>>4&1, byte>>3&1, byte>>2&1, byte>>1&1, byte>>0&1];
            let (r, g, b, a) = image::Pixel::channels4(&image::GenericImageView::get_pixel(&img, x, y));
            let r = r & 0b11111000 | bit_arr[0] << 2 | bit_arr[1] << 1 | bit_arr[2] << 0;
            let g = g & 0b11111100 | bit_arr[3] << 1 | bit_arr[4] << 0;
            let b = b & 0b11111000 | bit_arr[5] << 2 | bit_arr[6] << 1 | bit_arr[7] << 0;
            let a = a;
            let new_pixel = image::Pixel::from_channels(r, g, b, a);
            image::GenericImage::put_pixel(&mut img, x, y, new_pixel);
        }
    }
    image::DynamicImage::save(&img, "rights2.png");
    Ok(())
}
